@@ -290,6 +290,7 @@ pnprintf(char *string, size_t size, const char *f, va_list args)
     int p_len;
     int i;
     uint64_t ui;
+    int64_t si;
 
     for (; f_idx < f_len && s_idx < size - 1; f_idx++) {
         if (f[f_idx] != '%') {
@@ -311,6 +312,15 @@ pnprintf(char *string, size_t size, const char *f, va_list args)
             FormatUInt64(ui, number);
             p_len = strlen_sigsafe(number);
 
+            for (i = 0; i < p_len && s_idx < size - 1; i++)
+                string[s_idx++] = number[i];
+            break;
+        case 'i':
+        case 'd':
+            si = va_arg(args, int);
+            FormatInt64(si, number);
+            p_len = strlen_sigsafe(number);
+
             for (i = 0; i < p_len && s_idx < size - 1; i++)
                 string[s_idx++] = number[i];
             break;
@@ -364,7 +374,7 @@ LogSWrite(int verb, const char *buf, size_t len, Bool end_line)
     if (verb < 0 || logFileVerbosity >= verb) {
         if (inSignalContext && logFileFd >= 0) {
             write(logFileFd, buf, len);
-#ifdef WIN32
+#ifndef WIN32
             if (logFlush && logSync)
                 fsync(logFileFd);
 #endif
@@ -457,6 +467,7 @@ LogMessageTypeVerbString(MessageType type, int verb)
 void
 LogVMessageVerb(MessageType type, int verb, const char *format, va_list args)
 {
+    static unsigned int warned;
     const char *type_str;
     char buf[1024];
     const size_t size = sizeof(buf);
@@ -464,13 +475,17 @@ LogVMessageVerb(MessageType type, int verb, const char *format, va_list args)
     size_t len = 0;
 
     if (inSignalContext) {
-        BUG_WARN_MSG(inSignalContext,
-                     "Warning: attempting to log data in a signal unsafe "
-                     "manner while in signal context. Please update to check "
-                     "inSignalContext and/or use LogMessageVerbSigSafe() or "
-                     "ErrorFSigSafe(). The offending log format message is:\n"
-                     "%s\n", format);
-        return;
+        if (warned < 3) {
+            BUG_WARN_MSG(inSignalContext,
+                         "Warning: attempting to log data in a signal unsafe "
+                         "manner while in signal context.\nPlease update to check "
+                         "inSignalContext and/or use LogMessageVerbSigSafe() or "
+                         "ErrorFSigSafe().\nThe offending log format message is:\n"
+                         "%s\n", format);
+            warned++;
+            if (warned == 3)
+                LogMessageVerbSigSafe(X_WARNING, -1, "Warned %u times about sigsafe logging. Will be quiet now.\n", warned);
+        }
     }
 
     type_str = LogMessageTypeVerbString(type, verb);
@@ -556,6 +571,7 @@ void
 LogVHdrMessageVerb(MessageType type, int verb, const char *msg_format,
                    va_list msg_args, const char *hdr_format, va_list hdr_args)
 {
+    static unsigned int warned;
     const char *type_str;
     char buf[1024];
     const size_t size = sizeof(buf);
@@ -563,13 +579,17 @@ LogVHdrMessageVerb(MessageType type, int verb, const char *msg_format,
     size_t len = 0;
 
     if (inSignalContext) {
-        BUG_WARN_MSG(inSignalContext,
-                     "Warning: attempting to log data in a signal unsafe "
-                     "manner while in signal context. Please update to check "
-                     "inSignalContext and/or use LogMessageVerbSigSafe(). The "
-                     "offending header and log message formats are:\n%s %s\n",
-                     hdr_format, msg_format);
-        return;
+        if (warned < 3) {
+            BUG_WARN_MSG(inSignalContext,
+                         "Warning: attempting to log data in a signal unsafe "
+                         "manner while in signal context.\nPlease update to check "
+                         "inSignalContext and/or use LogMessageVerbSigSafe().\nThe "
+                         "offending header and log message formats are:\n%s %s\n",
+                         hdr_format, msg_format);
+            warned++;
+            if (warned == 3)
+                LogMessageVerbSigSafe(X_WARNING, -1, "Warned %u times about sigsafe logging. Will be quiet now.\n", warned);
+        }
     }
 
     type_str = LogMessageTypeVerbString(type, verb);
